@@ -35,6 +35,7 @@ class DymoAPI:
         self.server_email_config = config.get("server_email_config", None)
         self.rules = {
             "email": {"deny": ["FRAUD", "INVALID", "NO_MX_RECORDS", "NO_REPLY_EMAIL"]},
+            "ip": {"deny": ["FRAUD", "INVALID", "TOR_NETWORK"]},
             "phone": {"deny": ["FRAUD", "INVALID"]},
             "sensitive_info": {"deny": ["EMAIL", "PHONE", "CREDIT_CARD"]},
             **(config.get("rules") or {})
@@ -45,13 +46,13 @@ class DymoAPI:
         self.base_url = get_base_url()
         check_for_updates()
     
-    def _get_function(self, module_name, function_name="main"):
+    async def _get_function(self, module_name, function_name="main"):
         if module_name == "private" and self.api_key is None and self.root_api_key is None: return logging.error("Invalid private token.")
         func = getattr(importlib.import_module(f".branches.{module_name}", package="dymoapi"), function_name)
         if module_name == "private": return lambda *args, **kwargs: DotDict(func(self.api_key or self.root_api_key, *args, **kwargs))
         return lambda *args, **kwargs: DotDict(func(*args, **kwargs))
 
-    def is_valid_data(self, data: response_models.Validator) -> response_models.DataVerifierResponse:
+    async def is_valid_data(self, data: response_models.Validator) -> response_models.DataVerifierResponse:
         """
         Validates the given data against the configured validation settings.
 
@@ -78,7 +79,7 @@ class DymoAPI:
 
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/private/data-verifier)
         """
-        response = self._get_function("private", "is_valid_data")(data)
+        response = await self._get_function("private", "is_valid_data")(data)
         if response.get("ip",{}).get("as"):
             response["ip"]["_as"] = response["ip"]["as"]
             response["ip"]["_class"] = response["ip"]["class"]
@@ -86,7 +87,7 @@ class DymoAPI:
             response["ip"].pop("class")
         return response_models.DataVerifierResponse(**response)
     
-    def is_valid_email(self, email: str, rules: dict | None = None) -> bool:
+    async def is_valid_email(self, email: str, rules: dict | None = None) -> bool:
         """
         Wrapper for the private email validation function.
 
@@ -114,9 +115,39 @@ class DymoAPI:
             https://docs.tpeoficial.com/docs/dymo-api/private/email-validation
         """
         rules_to_use = rules or self.rules.get("email")
-        return self._get_function("private", "is_valid_email")(email, rules_to_use)
+        return await self._get_function("private", "is_valid_email")(email, rules_to_use)
     
-    def is_valid_phone(self, phone: str, rules: dict | None = None) -> bool:
+    async def is_valid_ip(self, ip: str, rules: dict | None = None) -> bool:
+        """
+        Wrapper for the private IP validation function.
+
+        Calls the internal `is_valid_ip` function with the provided IP and deny rules,
+        returning True or False according to the validation result.
+
+        Args:
+            ip (str): The IP address to validate.
+            rules (dict, optional): Validation rules object with key "deny" (list of deny rules). 
+                ⚠️ Some deny rules are PREMIUM: "TOR_NETWORK", "HIGH_RISK_SCORE".
+
+        Returns:
+            bool: True if the IP passes validation, False otherwise.
+
+        Raises:
+            APIError: If the underlying validation function fails or the API key is missing.
+
+        Example:
+            >>> valid = dymoClient.is_valid_ip(
+            >>>     "52.94.236.248",
+            >>>     rules={"deny": ["FRAUD", "TOR_NETWORK", "COUNTRY:RU"]}
+            >>> )
+
+        See also:
+            https://docs.tpeoficial.com/docs/dymo-api/private/ip-validation
+        """
+        rules_to_use = rules or self.rules.get("ip")
+        return await self._get_function("private", "is_valid_ip")(ip, rules_to_use)
+    
+    async def is_valid_phone(self, phone: str, rules: dict | None = None) -> bool:
         """
         Wrapper for the private phone validation function.
 
@@ -144,9 +175,9 @@ class DymoAPI:
             https://docs.tpeoficial.com/docs/dymo-api/private/phone-validation
         """
         rules_to_use = rules or self.rules.get("phone")
-        return self._get_function("private", "is_valid_phone")(phone, rules_to_use)
+        return await self._get_function("private", "is_valid_phone")(phone, rules_to_use)
     
-    def send_email(self, data: response_models.EmailStatus) -> response_models.SendEmailResponse:
+    async def send_email(self, data: response_models.EmailStatus) -> response_models.SendEmailResponse:
         """
         Sends an email using the configured email client settings.
 
@@ -179,9 +210,9 @@ class DymoAPI:
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/private/sender-send-email/getting-started)
         """
         if not self.server_email_config and not self.root_api_key: return logging.error("You must configure the email client settings.")
-        return response_models.DataVerifierResponse(**self._get_function("private", "send_email")({**data, "serverEmailConfig": self.server_email_config}))
+        return response_models.DataVerifierResponse(**await self._get_function("private", "send_email")({**data, "serverEmailConfig": self.server_email_config}))
     
-    def get_random(self, data: response_models.SRNG) -> response_models.SRNGResponse:
+    async def get_random(self, data: response_models.SRNG) -> response_models.SRNGResponse:
         """
         Generates a random number (or numbers) between the provided min and max values.
 
@@ -203,9 +234,9 @@ class DymoAPI:
             
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/private/secure-random-number-generator)
         """
-        return response_models.DataVerifierResponse(**self._get_function("private", "get_random")({**data}))
+        return response_models.DataVerifierResponse(**await self._get_function("private", "get_random")({**data}))
     
-    def extract_with_textly(self, data: response_models.Textly) -> response_models.TextlyResponse:
+    async def extract_with_textly(self, data: response_models.Textly) -> response_models.TextlyResponse:
         """
         Extracts structured data from a given text using the Textly endpoint.
 
@@ -227,9 +258,9 @@ class DymoAPI:
 
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/private/extract-textly)
         """        
-        return response_models.ExtractWithTextlyResponse(**self._get_function("private", "extract_with_textly")({**data}))
+        return response_models.ExtractWithTextlyResponse(**await self._get_function("private", "extract_with_textly")({**data}))
 
-    def get_prayer_times(self, data: response_models.PrayerTimesData) -> response_models.PrayerTimesResponse:
+    async def get_prayer_times(self, data: response_models.PrayerTimesData) -> response_models.PrayerTimesResponse:
         """
         Retrieves the prayer times for the given location.
 
@@ -253,9 +284,9 @@ class DymoAPI:
 
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/public/prayertimes)
         """
-        return response_models.PrayerTimesResponse(**self._get_function("public", "get_prayer_times")(data))
+        return response_models.PrayerTimesResponse(**await self._get_function("public", "get_prayer_times")(data))
 
-    def satinizer(self, data: response_models.InputSanitizerData) -> response_models.SatinizerResponse:
+    async def satinizer(self, data: response_models.InputSanitizerData) -> response_models.SatinizerResponse:
         """
         Sanitizes the input, replacing any special characters with their HTML entities.
 
@@ -273,9 +304,9 @@ class DymoAPI:
 
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/public/input-satinizer)
         """
-        return response_models.SatinizerResponse(**self._get_function("public", "satinizer")(data))
+        return response_models.SatinizerResponse(**await self._get_function("public", "satinizer")(data))
 
-    def is_valid_pwd(self, data: response_models.IsValidPwdData) -> response_models.IsValidPwdResponse:
+    async def is_valid_pwd(self, data: response_models.IsValidPwdData) -> response_models.IsValidPwdResponse:
         """
         Validates a password based on the given parameters.
 
@@ -308,9 +339,9 @@ class DymoAPI:
 
         [Documentation](https://docs.tpeoficial.com/docs/dymo-api/public/password-validator)
         """
-        return response_models.IsValidPwdResponse(**self._get_function("public", "is_valid_pwd")(data))
+        return response_models.IsValidPwdResponse(**await self._get_function("public", "is_valid_pwd")(data))
 
-    def new_url_encrypt(self, data) -> response_models.UrlEncryptResponse:
-        return response_models.UrlEncryptResponse(**self._get_function("public", "new_url_encrypt")(data))
+    async def new_url_encrypt(self, data) -> response_models.UrlEncryptResponse:
+        return response_models.UrlEncryptResponse(**await self._get_function("public", "new_url_encrypt")(data))
     
 if __name__ == "__main__": sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
\ No newline at end of file
